@@ -464,10 +464,21 @@ garcon_menu_set_property (GObject      *object,
  * @file: #GFile for the .menu file you want to load.
  *
  * Creates a new #GarconMenu for the .menu file referred to by @file.
- * This operation only fails @file is invalid. To load the menu
+ * This operation only fails if @file is invalid. To load the menu
  * tree from the file, you need to call garcon_menu_load() with the
  * returned #GarconMenu.
  *
+ * <informalexample><programlisting>
+ * GarconMenu *menu = garcon_menu_new (filename);
+ *
+ * if (garcon_menu_load (menu, &error))
+ *   ...
+ * else
+ *   ...
+ *
+ * g_object_unref (menu);
+ * </programlisting></informalexample>
+ *
  * The caller is responsible to destroy the returned #GarconMenu
  * using g_object_unref().
  *
@@ -489,21 +500,10 @@ garcon_menu_new (GFile *file)
  * @filename : Path/URI of the .menu file you want to load.
  *
  * Creates a new #GarconMenu for the .menu file referred to by @filename.
- * This operation only fails if the filename is NULL. To load the menu
+ * This operation only fails if the filename is %NULL. To load the menu
  * tree from the file, you need to call garcon_menu_load() with the
  * returned #GarconMenu.
  *
- * <informalexample><programlisting>
- * GarconMenu *menu = garcon_menu_new (filename);
- *
- * if (garcon_menu_load (menu, &error))
- *   ...
- * else
- *   ...
- *
- * g_object_unref (menu);
- * </programlisting></informalexample>
- *
  * The caller is responsible to destroy the returned #GarconMenu
  * using g_object_unref().
  *
@@ -2236,8 +2236,9 @@ garcon_menu_merge_dir_changed (GarconMenu       *menu,
 
 
 static gboolean
-garcon_menu_process_file_changes (GarconMenu *menu)
+garcon_menu_process_file_changes (gpointer user_data)
 {
+  GarconMenu *menu = user_data;
   GarconMenuItem *item;
   GFileType       file_type;
   gboolean        affects_the_outside = FALSE;
@@ -2381,7 +2382,7 @@ garcon_menu_app_dir_changed (GarconMenu       *menu,
             {
               garcon_menu_debug (NULL, 0, "schedule process file changes");
               menu->priv->file_changed_idle =
-                g_idle_add ((GSourceFunc) garcon_menu_process_file_changes, menu);
+                g_idle_add (garcon_menu_process_file_changes, menu);
             }
         }
     }
